"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const storage_1 = require("./storage");
exports.getChainHeight = async (db) => {
    const result = await db.one(storage_1.SQLs.getLastBlockHeight);
    return result.height;
};
const secondsToHumanString = (seconds) => {
    const years = Math.floor(seconds / 31536000);
    const days = Math.floor((seconds % 31536000) / 86400);
    const hours = Math.floor(((seconds % 31536000) % 86400) / 3600);
    const minutes = Math.floor((((seconds % 31536000) % 86400) % 3600) / 60);
    const numSeconds = (((seconds % 31536000) % 86400) % 3600) % 60;
    const result = [];
    if (years > 0) {
        result.push(`${years}y`);
    }
    if (days > 0) {
        result.push(`${days}d`);
    }
    if (hours > 0) {
        result.push(`${hours}h`);
    }
    if (minutes > 0) {
        result.push(`${minutes}m`);
    }
    if (numSeconds > 0) {
        result.push(`${numSeconds}s`);
    }
    if (result.length === 0) {
        return '0';
    }
    return result.join(' ');
};
const getRemainingTime = (currentHeight, observedHeight) => secondsToHumanString((observedHeight - currentHeight) * 10);
exports.observeChainHeight = async (options) => {
    const observedHeight = options.height;
    const startHeight = await exports.getChainHeight(options.db);
    if (startHeight === observedHeight) {
        return startHeight;
    }
    if (startHeight > observedHeight) {
        throw new Error(`Chain height: ${startHeight} crossed the observed height: ${observedHeight}`);
    }
    const progress = cli_ux_1.default.progress({
        format: `${options.label}: [{bar}] {percentage}% | Remaining: {remaining}/{total} | Height: {height}/${observedHeight} | ETA: {timeLeft}`,
        fps: 2,
        synchronousUpdate: false,
        etaAsynchronousUpdate: false,
        barsize: 30,
    });
    progress.start(observedHeight - startHeight, 0, {
        timeLeft: getRemainingTime(startHeight, observedHeight),
        remaining: observedHeight - startHeight,
        height: startHeight,
    });
    await new Promise((resolve, reject) => {
        let intervalId;
        const checkHeight = async () => {
            let height;
            try {
                height = await exports.getChainHeight(options.db);
            }
            catch (error) {
                return reject(error);
            }
            progress.update(height - startHeight, {
                timeLeft: getRemainingTime(height, observedHeight),
                remaining: observedHeight - height,
                height,
            });
            if (height === observedHeight) {
                clearInterval(intervalId);
                return resolve(height);
            }
            if (height > observedHeight) {
                return reject(new Error(`Chain height: ${height} crossed the observed height: ${observedHeight}`));
            }
        };
        intervalId = setInterval(checkHeight, options.delay);
    });
    progress.stop();
    return observedHeight;
};
//# sourceMappingURL=chain.js.map